searchData={"items":[{"type":"module","title":"TranslaTable","doc":"`TranslaTable` is a helper library for creating a translation schema inside your ecto schema.","ref":"TranslaTable.html"},{"type":"module","title":"Translation - TranslaTable","doc":"Inside your `config.exs` add your locale config\n\n    config :transla_table, :config, locale_schema: MyApp.Locale\n\n  To define a translation schema it just need to use the `TranslaTable` helper inside your schema to be translated\n    defmodule MyApp.Post do\n      use Ecto.Schema\n      use TranslaTable.Schema,\n        translation_schema: MyApp.PostTranslation\n\n      import Ecto.Changeset\n\n      alias MyApp.PostTranslation\n\n      schema \"post\" do\n        field :title, :string\n        field :description, :string\n        field :author, :string\n        field :slug, :string\n\n        has_many_translations()\n\n        timestamps()\n      end\n\n      @doc false\n      def changeset(post, attrs) do\n        post\n        |> cast(attrs, [:title, :description, :author, :slug])\n        |> cast_translation()\n      end\n    end\n\n  Then create the translation module\n\n    defmodule MyApp.PostTranslation\n      use TranslaTable,\n        schema: MyApp.Post,\n        fields: [:title, :description, :slug]\n    end\n\n  Then the TranslaTable Macro will create a schema with the given fields and make the relation with your `Locale` schema.","ref":"TranslaTable.html#module-translation"},{"type":"module","title":"TranslaTable.Query","doc":"Query helper when dealing internationalized data.","ref":"TranslaTable.Query.html"},{"type":"function","title":"TranslaTable.Query.assert_schema!/1","doc":"","ref":"TranslaTable.Query.html#assert_schema!/1"},{"type":"macro","title":"TranslaTable.Query.localize_query/2","doc":"Create a query to localize an entity from database.","ref":"TranslaTable.Query.html#localize_query/2"},{"type":"macro","title":"Example - TranslaTable.Query.localize_query/2","doc":"You can create your own functions for composing queries.\n\n    defmodule MyApp.Blog.Post do\n      import Ecto.Query\n      import TranslaTable.Query\n      alias MyApp.Post\n      alias MyApp.Repo\n\n      def run(filters) do\n        base_query()\n        |> localize_query(filters[\"locale\"])\n        |> filter_by_title(filters)\n        |> Repo.all()\n      end\n\n      def base_query() do\n        from b in Post\n      end\n\n      def filter_by_title(query, %{title: title}) do\n        from b in query,\n        where: title == ^title\n      end\n\n      def filter_by_title(query, _), do: query\n    end\n\n  The `run` method will return your data translated based on the `locale` passed in the filters.\n  This `locale` parameter should be the reference id of your Locale table.\n\n  The method `filter_by_title` is not filtering by the localized value.\n  If you want all your filters to be localized, It's recomended to structure your module in this way:\n\n    defmodule MyApp.Blog.Post do\n      import Ecto.Query\n      import TranslaTable.Query\n      alias MyApp.Post\n      alias MyApp.Repo\n\n      def run(filters) do\n        base_query()\n        |> localize(filters)\n        |> filter_by_title(filters)\n        |> Repo.all()\n      end\n\n      def base_query() do\n        from b in Post\n      end\n\n      def localize(query, %{locale: locale}) do\n        localize_query(query, locale)\n      end\n\n      def localize(query, _params), do: query\n\n      def filter_by_title(query, %{title: title, locale: _}) do\n        from [p, translations: tr] in query, # This is the name binding to the translations (joined by the localize method)\n          where: not is_nil(tr.title) and tr.title == ^title,\n          or_where: not is_nil(p.title) and p.title == ^title\n      end\n\n      # This will be the default method when the locale is not added as an argument\n      def filter_by_title(query, %{title: title}) do\n        from b in query,\n        where: title == ^title\n      end\n\n      def filter_by_title(query, _), do: query\n    end\n\n  This way the queries are composable by locale and It's flexible enough for not using the locale argument at all.","ref":"TranslaTable.Query.html#localize_query/2-example"},{"type":"module","title":"TranslaTable.Schema","doc":"`TranslaTable.Schema` will help casting translation module into your schema.","ref":"TranslaTable.Schema.html"},{"type":"module","title":"Examples - TranslaTable.Schema","doc":"You can define your schema module with the `TranslaTable.Schema` macro to cast and define the has many association:\n    defmodule MyApp.Post do\n\n    use Ecto.Schema\n    use TranslaTable.Schema,\n      translation_schema: MyApp.PostTranslation\n\n    import Ecto.Changeset\n\n    schema \"post\" do\n      field :title, :string\n      field :description, :string\n      field :author, :string\n      field :slug, :string\n\n      has_many_translations()\n\n      timestamps()\n    end\n\n    @doc false\n    def changeset(post, attrs) do\n      post\n      |> cast(attrs, [:title, :description, :author, :slug])\n      |> cast_translation()\n    end\n  end","ref":"TranslaTable.Schema.html#module-examples"},{"type":"macro","title":"TranslaTable.Schema.cast_translation/1","doc":"","ref":"TranslaTable.Schema.html#cast_translation/1"},{"type":"macro","title":"TranslaTable.Schema.has_many_translations/0","doc":"","ref":"TranslaTable.Schema.html#has_many_translations/0"},{"type":"extras","title":"TranslaTable","doc":"# TranslaTable\n- [TranslaTable](#translatable)\n  - [Installation](#installation)\n  - [Locale Schema](#locale-schema)\n    - [Creating locale migration and schema](#creating-locale-migration-and-schema)\n    - [Creating migration for the entity to be translated](#creating-migration-for-the-entity-to-be-translated)\n  - [Translate Schemas](#translate-schemas)\n  - [Querying data](#querying-data)\n  - [License](#license)\n\nTranslaTable is a thin wrapper around [Ecto](https://hexdocs.pm/ecto/Ecto.html) library to provide internationalizable entities and a way to query localized data.\nSee the documentation on [HexDocs](https://hexdocs.pm/transla_table).","ref":"readme.html"},{"type":"extras","title":"Installation - TranslaTable","doc":"Add the `TranslaTable` package in your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:transla_table, \"~> 0.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Locale Schema - TranslaTable","doc":"You need to create a `Locale` table and schema to be used and define the locales available for translation.","ref":"readme.html#locale-schema"},{"type":"extras","title":"Creating locale migration and schema - TranslaTable","doc":"```elixir\ndefmodule MyApp.CreateLocale do\n  use Ecto.Migration\n\n  def change do\n    create table(:locale, primary_key: false) do\n      add :id, :string, size: 10, primary_key: true # added as string just to reference the primary key as \"en\", \"es\", \"pt\", etc\n      add :name, :string, null: false\n      timestamps()\n    end\n  end\nend\n\n```\n\n```elixir\ndefmodule MyApp.Locale do\n  @moduledoc \"\"\"\n  Locale Schema\n  \"\"\"\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  @primary_key {:id, :string, autogenerate: false}\n  schema \"locale\" do\n    field :name, :string\n    timestamps()\n  end\nend\n```\n\nThen Inside your `config.exs` add your locale config\n\n```Elixir\n\nconfig :transla_table, :config, locale_schema: MyApp.Locale\n\n```","ref":"readme.html#creating-locale-migration-and-schema"},{"type":"extras","title":"Creating migration for the entity to be translated - TranslaTable","doc":"For example, if you have a `Post` table and want it to be internationalized, create the table which will make the relation between `post` and `locale`:\n\n\n```elixir\ndefmodule MyApp.CreatePostTranslation do\n  use Ecto.Migration\n\n  def change do\n    create table(:post_translation) do\n      add :post_id, references(:post, on_delete: :delete_all)\n      add :locale_id, references(:locale, type: :string, on_delete: :delete_all)\n\n      # fields to translate in the Post table\n      add :title, :string\n      add :description, :string\n      add :slug, :string\n\n      timestamps()\n    end\n    create unique_index(:post_translation, [:post_id, :locale_id])\n  end\nend\n\n```","ref":"readme.html#creating-migration-for-the-entity-to-be-translated"},{"type":"extras","title":"Translate Schemas - TranslaTable","doc":"Then in your `Post` module you define the translation methods using the `TranslaTable` helper inside your schema to be translated\n\n```elixir\ndefmodule MyApp.Post do\n  use Ecto.Schema\n  use TranslaTable.Schema,\n    translation_schema: MyApp.PostTranslation\n\n  import Ecto.Changeset\n\n  schema \"post\" do\n    field :title, :string\n    field :description, :string\n    field :author, :string\n    field :slug, :string\n\n    has_many_translations()\n\n    timestamps()\n  end\n\n  @doc false\n  def changeset(post, attrs) do\n    post\n    |> cast(attrs, [:title, :description, :author, :slug])\n    |> cast_translation()\n  end\nend\n```\n\nFinally create the translation module which will automatically map the fields within the `Post` schema\n\n```elixir\ndefmodule MyApp.PostTranslation\n  use TranslaTable,\n    schema: MyApp.Post,\n    fields: [:title, :description, :slug]\nend\n```\nthen this will create a Translation module with the relations and fields.","ref":"readme.html#translate-schemas"},{"type":"extras","title":"Querying data - TranslaTable","doc":"Using the same example above, it is possible to query data to return only localized fields.\n\n```Elixir\ndefmodule MyApp.PostContext do\n  import Ecto.Query\n  import TranslaTable.Query\n  \n  alias MyApp.Post\n  alias MyApp.Repo\n\n  def list_all() do\n    from(p in Post)\n    |> preload(:translations)\n    |> Repo.all()\n  end\n\n  def list_localized(locale_id) do\n    from(p in Post)\n    |> localize_query(locale_id)\n    |> Repo.all()\n  end\nend\n\n...","ref":"readme.html#querying-data"},{"type":"extras","title":"then you can list the posts with translations - TranslaTable","doc":"iex> MyApp.PostContext.list_all()\n[\n  %MyApp.Post{\n    description: \"Description\",\n    title: \"Blog Post\",\n    translations: [\n      %MyApp.PostTranslation{\n        description: \"Description\",\n        title: \"Blog Post\"\n      },\n      %MyApp.PostTranslation{\n        description: \"Descrição\",\n        title: \"Post do Blog\"\n      }\n    ]\n  }\n]\n\n# also you can define which locale you want to return\niex> MyApp.PostContext.list_localized(\"pt\")\n[\n  %MyApp.Post{\n    description: \"Descrição\",\n    title: \"Post do Blog\"\n  }\n]\n```","ref":"readme.html#then-you-can-list-the-posts-with-translations"},{"type":"extras","title":"License - TranslaTable","doc":"Copyright 2021 Paulo Henrique de Oliveira Curado\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}